@@ -3,9 +3,6 @@
 
 #include "common.h"
 
-#include <assert.h>
-#include <stdbool.h>
-
 #include "input_events.h"
 
 /**
